#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <libgnome/gnome-i18n.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "interface.h"
#include "support.h"
#include "callbacks.h"
#include "coda.h"
#include "docklet.h"


// check if language is supported
gboolean lingua_supportata(gchar *l);

// adds table to interface, not in interface.c because not supported by glade!
void
setup_tree (void);

// check parameters, return 1 if "param" is in "argv"
int 
check_parameter (int argc, char *argv[], char *param, char *valore);



// Main
int
main (int argc, char *argv[])
{
  GtkWidget *gqueue;
  int parametri=0,mypid;
  gchar *lingua,*lock_file_name,*myhome,*command;
  pid_t child;
  FILE *lock_file_descriptor;
  
  
  // Swich to block event functions in some cases, 0 = not blocked
  semaforo=0;
  semaforo_window_state_event=0;
  
  // --help
  if(argc>1)
  {
	  if(!strcmp("--help",argv[1]))
	  {
		  printf("Usage: gqueue [OPTION...]\n");
		  printf("  --LANGUAGE=xx                          Force use xx language\n");
		  printf("                                         xx = [it|fr|es|en|de|pt|ru|nl|et|pl|no]\n");
		  printf("  --no_system_tray                       Disable system tray support\n\n");
	  }
  }
	
  
  // initialize gettext and gnome
  #ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (PACKAGE);
  #endif
 
  gnome_program_init (PACKAGE, "0.8", LIBGNOMEUI_MODULE,
                      argc, argv,
                      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
                      NULL);
  
  
  // Go in background
  child = fork();
  if (child  > 0)
  {
	  exit(0);	// Parent exits
  }
  
  // create a lock file
  myhome = getenv ("HOME");
  mypid=getpid();
  lock_file_name = g_strdup_printf ("%s/%s.%d", myhome, ".gqueue-lock",mypid);
  command = g_strdup_printf ("%s %s/%s","rm -rf",myhome, ".gqueue-lock.*");
  if(system(command)==-1)
  {
	printf("Error executing a script\n");  
  }
  if((lock_file_descriptor=fopen(lock_file_name,"w+"))==NULL)
	{
		// lock not exist then exit
		printf("Error creating a lock file\n");
	}
  else fclose(lock_file_descriptor);
  g_free(myhome);
  g_free(lock_file_name);
  g_free(command);
	
  // Load preferences and set variables
  gtk_set_locale();
  gqueuepref_read (&pref);
  system_tray=pref.system_tray;
  
  // Check and set parameters 
  lingua = g_strdup_printf ("%s","valore"); // init of var lingua
  if(check_parameter (argc,argv,"--LANGUAGE=",lingua)) 
  {
	  if(lingua_supportata(lingua))
	  {
		  strncpy(pref.lang,lingua,2);
		  pref.lang[2]='\0';
		  printf("Force language to %s.\n",pref.lang);
		  setenv("LANGUAGE",pref.lang,1);
		  gqueuepref_write (&pref);
	  }
	  else
	  {
	      printf("Language '%s' not supported.\nSet to ENV variable.\n",lingua);
	  }
  }
  
  if(check_parameter (argc,argv,"--no_system_tray",NULL))
  {
	  pref.system_tray=0;
	  system_tray=0;
	  printf("System tray support off.\n");
	  gqueuepref_write (&pref);
  }
  
  if(check_parameter (argc,argv,"--no-system-tray",NULL))
  {
	  pref.system_tray=0;
	  system_tray=0;
	  printf("System tray support off.\n");
	  gqueuepref_write (&pref);
  }

  /* Operazioni di inizializzazione varie - init operations */
  gqueue = create_gqueue ();
  gtk_widget_show (gqueue);
  FinestraPrincipale=gqueue;
  setup_tree();
  combo_init();
	    
  /* init system tray */
  if(system_tray==1)
  {
	  docklet_register_icon_factory();
	  docklet_create(gqueue);
	  docklet_toggle(gqueue);
  }	

  
  // Fix menu item "in_system_tray" value
  semaforo=1;
  gtk_check_menu_item_set_active(
	  GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(FinestraPrincipale),"in_system_tray")),
	  system_tray);
  semaforo=0;
  

  /* Automatic queue refresh every 2000 millisecond */
   
  gtk_timeout_add( 2000,
                 queue_refresh,
                 NULL );
					  
  /* Inizio del ciclo principale di gtk - Start of gtk cicle */
  gtk_main ();
  return 0;
}


// create table of printing items
void
setup_tree (void)
{
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
	
   GtkTreeIter   iter;  


   store = gtk_list_store_new (N_COLUMNS,
                               G_TYPE_STRING,
                               G_TYPE_INT,
	                           G_TYPE_STRING,
                               G_TYPE_STRING,
	                           G_TYPE_STRING);


   /* Create a view */
   tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));

   /* The view now holds a reference.  We can get rid of our own
    * reference */
   g_object_unref (G_OBJECT (store));

   /* Create a column, associating the "text" attribute of the
    * cell_renderer to the first column of the model */
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (gettext("Stampante"), renderer,
                                                      "text", STAMPANTE_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column,TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

   /* Second column. */
   column = gtk_tree_view_column_new_with_attributes (gettext("ID"),renderer,
                                                      "text", ID_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column,TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

   /* III column. */
   column = gtk_tree_view_column_new_with_attributes (gettext("Utente"),renderer,
                                                      "text", UTENTE_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column,TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
   
   /* IV column. */
   column = gtk_tree_view_column_new_with_attributes (gettext("Titolo"),renderer,
                                                      "text", TITOLO_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column,TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
   
   /* Last column. */
   column = gtk_tree_view_column_new_with_attributes (gettext("Stato"),renderer,
                                                      "text", STATO_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column,TRUE);												
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
   
   
  
  gtk_widget_set_name (tree, "tree");
  gtk_widget_show (tree);
   
  gtk_container_add (GTK_CONTAINER (lookup_widget ( GTK_WIDGET(FinestraPrincipale), 
   						"scrolledwindow1" )), tree);

							   
}

// read gQueue preferences
void gqueuepref_read (gqueuepref *ppref)
	{
	FILE *file_pointer;
	gchar *home_dir, *pref_file_with_path, *s, *langenv;
	gint i, v;
	gqueuepref qpref;

	/*
	 * set default preferences
	 */

	// system tray off
	qpref.system_tray = 0;
	// environment language, no force lo language
	strcpy(qpref.lang,"");
	
	/*
	 * try to open preferences file in read mode
	 */

	home_dir = getenv ("HOME");
	pref_file_with_path = g_strdup_printf ("%s/%s", home_dir, ".gqueue");
	if ((file_pointer = fopen (pref_file_with_path, "r")) != NULL)
	{
		s = g_malloc (MAX_STRING_LENGTH + 2);

		for (i = 0; i <= 0; i++)
			if (((s = fgets (s, MAX_STRING_LENGTH + 2, file_pointer))
				!= NULL) && (s[0] != '\n'))
			{
				s[strlen (s) - 1] = 0;
				v = atoi (s);
				if ((v == 0) || (v == 1))
				{
					switch (i)
					{
						case 0 : qpref.system_tray = v; continue;
					}
				}
			}
		if (((s = fgets (s, MAX_STRING_LENGTH + 2, file_pointer)) != NULL) && (s[0] != '\n'))
		{
			s[strlen (s) - 1] = 0;
			printf("Language is %s\n",s);
			strcpy(qpref.lang,s);
			setenv("LANGUAGE",qpref.lang,1);
		}

		fclose (file_pointer);
		g_free (s);
	}
	g_free (home_dir);
	g_free (pref_file_with_path);
	
	/*
	 * call by reference
	 */	

	*ppref = qpref;
	return;
}


// Save pref.
void gqueuepref_write (const gqueuepref *pref)
{
	FILE *file_pointer;
	gchar *home_dir, *pref_file_with_path, *tmp;

	/*
	 * try to open preferences file in (over)writing mode
	 */

	home_dir = getenv ("HOME");
	pref_file_with_path = g_strdup_printf ("%s/%s", home_dir, ".gqueue");
	if ((file_pointer = fopen (pref_file_with_path, "w+")) != NULL)
	{
		tmp = g_strdup_printf (
			"%d\n%s\n",
			pref->system_tray,
		    pref->lang);
		//printf("Scrivo le pref con lingua=%s\n",pref->lang);
		fputs (tmp, file_pointer);	
		fclose (file_pointer);
	}
	g_free (home_dir);
	g_free (pref_file_with_path);
	return;
}


int 
check_parameter (int argc, char *argv[], char *param, char *valore)
// valore must point to an allocated memory area
{
	int i,j=0,max,valore_lenght;
	
	if(valore==NULL) valore_lenght=0;
	else valore_lenght=strlen(valore);
	
	for(i=1;i<argc;i++)
	{
		if(strlen(param)<=strlen(argv[i]))
		{
			if(!strncmp(argv[i],param,strlen(param)))
			{
				if(valore!=NULL)
			    {
					while(argv[i][j+strlen(param)]!=0)
				    {
					    if(j<valore_lenght)
					    {
						    valore[j]=argv[i][j+strlen(param)];
					        j++;
					    }
				    }
				    valore[j]=0;
					return(1);
			    }
			    else
				{
					if(strlen(param)==strlen(argv[i])) return(1);
				}
		    }
	    }
	}
	return(0);
}


gboolean lingua_supportata(gchar *l)
{
	
	if(!strcmp(l,"it")) return TRUE; 
	if(!strcmp(l,"en")) return TRUE; 
	if(!strcmp(l,"fr")) return TRUE; 
	if(!strcmp(l,"es")) return TRUE; 
	if(!strcmp(l,"ru")) return TRUE; 
	if(!strcmp(l,"de")) return TRUE; 
	if(!strcmp(l,"pt")) return TRUE; 
	if(!strcmp(l,"nl")) return TRUE;
	if(!strcmp(l,"et")) return TRUE;
	if(!strcmp(l,"pl")) return TRUE;
	if(!strcmp(l,"no")) return TRUE;

	return FALSE;
	
}
